unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Data.DB,
  Vcl.DBCtrls, Datasnap.DBClient, Vcl.Grids, Vcl.DBGrids, Vcl.ExtCtrls,
  Vcl.StdCtrls, Vcl.Mask, Vcl.Buttons, ShellApi;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    DBGrid1: TDBGrid;
    Panel2: TPanel;
    cdsUfo: TClientDataSet;
    dsUfo: TDataSource;
    DBNavigator1: TDBNavigator;
    DBMemo1: TDBMemo;
    panEdit: TPanel;
    edID: TDBEdit;
    edDesc: TDBEdit;
    edLocation: TDBEdit;
    edCountry: TDBEdit;
    edCE: TDBEdit;
    edSource: TDBEdit;
    edDurat: TDBEdit;
    edWit: TDBEdit;
    edShape: TDBEdit;
    edAppear: TDBEdit;
    edSize: TDBEdit;
    edOcc: TDBEdit;
    btnSave: TBitBtn;
    btnCancel: TBitBtn;
    edSearch: TEdit;
    btnSearch: TSpeedButton;
    edLink2: TDBEdit;
    btnWeb: TSpeedButton;
    lblURL: TLabel;
    cdsUfo2: TClientDataSet;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure DBNavigator1Click(Sender: TObject; Button: TNavigateBtn);
    procedure btnSaveClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnSearchClick(Sender: TObject);
    procedure edSearchKeyPress(Sender: TObject; var Key: Char);
    procedure btnWebClick(Sender: TObject);
    procedure cdsUfoBeforeEdit(DataSet: TDataSet);
    procedure cdsUfoBeforeInsert(DataSet: TDataSet);
    procedure cdsUfoAfterCancel(DataSet: TDataSet);
    procedure cdsUfoAfterPost(DataSet: TDataSet);
    procedure cdsUfoAfterDelete(DataSet: TDataSet);
  private
    { Private declarations }
    XmlFile: string;
    DataChanged: Boolean;
    OldID: string;
    procedure UpdateGUI(Modify: Boolean);
    function IsValid: Boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  DataChanged := False;
  XmlFile := 'ufodata.xml';
  if FileExists(XmlFile) then
    cdsUfo.LoadFromFile(XmlFile)
  else begin
    cdsUfo.FieldDefs.Add('ID',       ftString, 10, False);
    cdsUfo.FieldDefs.Add('DESC',     ftString, 25, False);
    cdsUfo.FieldDefs.Add('LOCATION', ftString, 16, False);
    cdsUfo.FieldDefs.Add('COUNTRY',  ftString, 16, False);
    cdsUfo.FieldDefs.Add('CE',       ftString,  1, False);
    cdsUfo.FieldDefs.Add('SOURCE',   ftString, 25, False);
    cdsUfo.FieldDefs.Add('DURAT',    ftString, 16, False);
    cdsUfo.FieldDefs.Add('WIT',      ftString,  5, False);
    cdsUfo.FieldDefs.Add('SHAPE',    ftString, 10, False);
    cdsUfo.FieldDefs.Add('APPEAR',   ftString, 10, False);
    cdsUfo.FieldDefs.Add('SIZE',     ftString, 10, False);
    cdsUfo.FieldDefs.Add('OCC',      ftString,  3, False);
    cdsUfo.FieldDefs.Add('LINK2',    ftString, 50, False);
    cdsUfo.FieldDefs.Add('NOTES',    ftMemo,   10, False);
    cdsUfo.CreateDataSet;
    DataChanged := True;
  end;
  cdsUfo.LogChanges := False;
  panEdit.Visible := False;
  lblURL.Caption := '';
end;

procedure TForm1.btnSaveClick(Sender: TObject);
begin
  if IsValid then
    cdsUfo.Post;
end;

function TForm1.IsValid: Boolean;
var
  ID: string;
begin
  ID := edID.Text;
  Result := Length(Trim(ID)) > 0;
  if not Result then
    ShowMessage('Field ID is empty. Please, enter something.')
  else if (cdsUfo.State = dsInsert) or
     ((cdsUfo.State = dsEdit) and
      (ID <> OldID)) then begin
    cdsUfo2.CloneCursor(cdsUfo, False, False);
    cdsUfo2.Open;
    cdsUfo2.IndexFieldNames := 'ID';
    Result := not cdsUfo2.FindKey([ID]);
    cdsUfo2.Close;
    if not Result then
      ShowMessage('Double ID. Please, enter another ID.');
  end;
end;

procedure TForm1.btnSearchClick(Sender: TObject);
var
  IsFound: Boolean;
  ID, Desc, Locat: string;
begin
  ID := cdsUfo.FieldByName('ID').AsString;
  cdsUfo.DisableControls;
  cdsUfo.First;
  IsFound := False;
  while (not cdsUfo.EOF) and (not IsFound) do begin
    Desc := UpperCase(cdsUfo.FieldByName('DESC').AsString);
    Locat := UpperCase(cdsUfo.FieldByName('LOCATION').AsString);
    IsFound :=
      (Pos(UpperCase(edSearch.Text), Desc) > 0) or
      (Pos(UpperCase(edSearch.Text), Locat) > 0);
    if not IsFound then
      cdsUfo.Next;
  end;
  if IsFound then
    DBGrid1.SetFocus
  else
    cdsUFO.FindKey([ID]);
    cdsUfo.EnableControls;
end;

procedure TForm1.btnWebClick(Sender: TObject);
var
  URL: string;
begin
  lblURL.Caption := '';
  URL := Trim(cdsUfo.FieldByName('LINK2').AsString);
  if Length(URL) > 0 then begin
    lblURL.Caption := URL;
    ShellExecute(handle, nil, PChar(URL), nil, nil, SW_SHOWNORMAL);
  end;
end;

procedure TForm1.UpdateGUI(Modify: Boolean);
begin
  DBGrid1.Enabled := not Modify;
  DBNavigator1.Enabled := not Modify;
  edSearch.Enabled := not Modify;
  btnSearch.Enabled := not Modify;
  btnWeb.Enabled := not Modify;
  panEdit.Visible := Modify;
  if Modify then
    edID.SetFocus
  else
    DBGrid1.SetFocus;
end;

procedure TForm1.cdsUfoAfterCancel(DataSet: TDataSet);
begin
  UpdateGUI(False);
end;

procedure TForm1.cdsUfoAfterDelete(DataSet: TDataSet);
begin
  DataChanged := True;
end;

procedure TForm1.cdsUfoAfterPost(DataSet: TDataSet);
begin
  DataChanged := True;
  UpdateGUI(False);
end;

procedure TForm1.cdsUfoBeforeEdit(DataSet: TDataSet);
begin
  OldID := cdsUfo.FieldByName('ID').AsString;
  UpdateGUI(True);
end;

procedure TForm1.cdsUfoBeforeInsert(DataSet: TDataSet);
begin
  UpdateGUI(True);
end;

procedure TForm1.btnCancelClick(Sender: TObject);
begin
  cdsUfo.Cancel;
end;

procedure TForm1.DBNavigator1Click(Sender: TObject; Button: TNavigateBtn);
begin
  if (Button = nbEdit) or (Button = nbInsert) then
    panEdit.Visible := True;
end;

procedure TForm1.edSearchKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
    btnSearch.Click;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if DataChanged then
    cdsUfo.SaveToFile(XmlFile, dfXML);
end;

end.
